@@ -14,7 +14,6 @@
 
 import os
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_service import loopingcall
 
@@ -25,8 +24,6 @@ from os_brick.initiator.connectors import base
 from os_brick.initiator import linuxfc
 from os_brick import utils
 
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
-
 LOG = logging.getLogger(__name__)
 
 
@@ -168,7 +165,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return volume_paths
 
     @utils.trace
-    @synchronized('extend_volume', external=True)
+    @base.synchronized('extend_volume', external=True)
     @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
@@ -191,7 +188,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
 
     @utils.trace
     @utils.connect_volume_prepare_result
-    @synchronized('connect_volume', external=True)
+    @base.synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties):
         """Attach the volume to instance_name.
 
@@ -316,7 +313,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return raw_devices
 
     @utils.trace
-    @synchronized('connect_volume', external=True)
+    @base.synchronized('connect_volume', external=True)
     @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
@@ -332,7 +329,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         target_wwn - World Wide Name
         target_lun - LUN id of the volume
         """
-        exc = exception.ExceptionChainer()
+
         devices = []
         wwn = None
 
@@ -348,30 +345,14 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                 wwn = self._linuxscsi.get_scsi_wwn(path)
                 mpath_path = self._linuxscsi.find_multipath_device_path(wwn)
                 if mpath_path:
-                    with exc.context(force, 'Flushing %s failed', mpath_path):
-                        self._linuxscsi.flush_multipath_device(mpath_path)
+                    self._linuxscsi.flush_multipath_device(mpath_path)
             dev_info = self._linuxscsi.get_device_info(real_path)
             devices.append(dev_info)
 
-        # If flush failed, then remove it forcefully since force=True
-        if mpath_path and exc:
-            with exc.context(force, 'Removing multipath %s failed',
-                             mpath_path):
-                mpath_name = os.path.basename(os.path.realpath(mpath_path))
-                self._linuxscsi.multipath_del_map(mpath_name)
-
         LOG.debug("devices to remove = %s", devices)
-        self._remove_devices(connection_properties, devices, device_info,
-                             force, exc)
+        self._remove_devices(connection_properties, devices, device_info)
 
-        if exc:  # type: ignore
-            LOG.warning('There were errors removing %s, leftovers may remain '
-                        'in the system', volume_paths)
-            if not ignore_errors:
-                raise exc  # type: ignore
-
-    def _remove_devices(self, connection_properties, devices, device_info,
-                        force, exc):
+    def _remove_devices(self, connection_properties, devices, device_info):
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
@@ -386,12 +367,11 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # paths, whereas for multipaths we have multiple link formats.
         was_multipath = '/pci-' not in path_used and was_symlink
         for device in devices:
-            with exc.context(force, 'Removing device %s failed', device):
-                device_path = device['device']
-                flush = self._linuxscsi.requires_flush(device_path,
-                                                       path_used,
-                                                       was_multipath)
-                self._linuxscsi.remove_scsi_device(device_path, flush=flush)
+            device_path = device['device']
+            flush = self._linuxscsi.requires_flush(device_path,
+                                                   path_used,
+                                                   was_multipath)
+            self._linuxscsi.remove_scsi_device(device_path, flush=flush)
 
     def _get_pci_num(self, hba):
         # NOTE(walter-boring)
