@@ -21,7 +21,6 @@ import re
 import time
 from typing import List, Tuple  # noqa: H301
 
-from oslo_concurrency import lockutils
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -36,8 +35,6 @@ from os_brick.initiator.connectors import base_iscsi
 from os_brick.initiator import utils as initiator_utils
 from os_brick import utils
 
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
-
 LOG = logging.getLogger(__name__)
 
 
@@ -474,7 +471,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             root_helper=self._root_helper)
 
     @utils.trace
-    @synchronized('extend_volume')
+    @base.synchronized('extend_volume', external=True)
+    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties: dict):
         """Update the local kernel's size information.
 
@@ -497,18 +495,11 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             raise exception.VolumePathsNotFound()
 
     @utils.trace
-    @synchronized('connect_volume')
+    @utils.connect_volume_prepare_result
+    @base.synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties: dict):
         """Attach the volume to instance_name.
 
-        NOTE: Will retry up to three times to handle the case where c-vol
-        and n-cpu are both using os-brick to manage iSCSI sessions but they
-        are on the same node and using different locking directories. In this
-        case, even though this call is synchronized, they will be separate
-        locks and can still overlap with connect and disconnect. Since a
-        disconnect during an initial attach can't cause IO failure (the device
-        has not been made available yet), we just try the connection again.
-
         :param connection_properties: The valid dictionary that describes all
                                       of the target volume attributes.
         :type connection_properties: dict
@@ -531,47 +522,14 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             with excutils.save_and_reraise_exception():
                 self._cleanup_connection(connection_properties, force=True)
 
-    @utils.retry(exceptions=(exception.VolumeDeviceNotFound))
-    def _get_device_link(self, wwn, device, mpath):
-        # These are the default symlinks that should always be there
-        if mpath:
-            symlink = '/dev/disk/by-id/dm-uuid-mpath-' + mpath
-        else:
-            symlink = '/dev/disk/by-id/scsi-' + wwn
-
-        # If default symlinks are not there just search for anything that links
-        # to our device.  In my experience this will return the last added link
-        # first, so if we are going to succeed this should be fast.
-        if not os.path.realpath(symlink) == device:
-            links_path = '/dev/disk/by-id/'
-            for symlink in os.listdir(links_path):
-                symlink = links_path + symlink
-                if os.path.realpath(symlink) == device:
-                    break
-            else:
-                # Raising this will trigger the next retry
-                raise exception.VolumeDeviceNotFound(device='/dev/disk/by-id')
-        return symlink
-
     def _get_connect_result(self, con_props, wwn, devices_names, mpath=None):
         device = '/dev/' + (mpath or devices_names[0])
-
-        # NOTE(geguileo): This is only necessary because of the current
-        # encryption flow that requires that connect_volume returns a symlink
-        # because first we do the volume attach, then the libvirt config is
-        # generated using the path returned by the atach, and then we do the
-        # encryption attach, which is forced to preserve the path that was used
-        # in the libvirt config.  If we fix that flow in OS-brick, Nova, and
-        # Cinder we can remove this and just return the real path.
-        if con_props.get('encrypted'):
-            device = self._get_device_link(wwn, device, mpath)
-
         result = {'type': 'block', 'scsi_wwn': wwn, 'path': device}
         if mpath:
             result['multipath_id'] = wwn
         return result
 
-    @utils.retry(exceptions=(exception.VolumeDeviceNotFound))
+    @utils.retry((exception.VolumeDeviceNotFound))
     def _connect_single_volume(self, connection_properties):
         """Connect to a volume using a single path."""
         data = {'stop_connecting': False, 'num_logins': 0, 'failed_logins': 0,
@@ -698,7 +656,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             data['just_added_devices'].append(device)
         data['stopped_threads'] += 1
 
-    @utils.retry(exceptions=(exception.VolumeDeviceNotFound))
+    @utils.retry((exception.VolumeDeviceNotFound))
     def _connect_multipath_volume(self, connection_properties):
         """Connect to a multipathed volume launching parallel login requests.
 
@@ -870,7 +828,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         return device_map
 
     @utils.trace
-    @synchronized('connect_volume')
+    @base.synchronized('connect_volume', external=True)
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Detach the volume from instance_name.
@@ -936,8 +895,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         for remove, __ in devices_map.values():
             remove_devices.update(remove)
 
-        path_used = self._linuxscsi.get_dev_path(connection_properties,
-                                                 device_info)
+        path_used = utils.get_dev_path(connection_properties, device_info)
         was_multipath = (path_used.startswith('/dev/dm-') or
                          'mpath' in path_used)
         multipath_name = self._linuxscsi.remove_connection(
@@ -1051,6 +1009,17 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         return ips, iqns
 
     def _connect_to_iscsi_portal(self, connection_properties):
+        """Safely connect to iSCSI portal-target and return the session id."""
+        portal = connection_properties['target_portal'].split(",")[0]
+        target_iqn = connection_properties['target_iqn']
+
+        lock_name = f'connect_to_iscsi_portal-{portal}-{target_iqn}'
+        method = base.synchronized(
+            lock_name, external=True)(self._connect_to_iscsi_portal_unsafe)
+        return method(connection_properties)
+
+    @utils.retry((exception.BrickException))
+    def _connect_to_iscsi_portal_unsafe(self, connection_properties):
         """Connect to an iSCSI portal-target an return the session id."""
         portal = connection_properties['target_portal'].split(",")[0]
         target_iqn = connection_properties['target_iqn']
@@ -1066,9 +1035,17 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         out, err = self._run_iscsiadm(connection_properties, (),
                                       check_exit_code=(0, 21, 255))
         if err:
-            self._run_iscsiadm(connection_properties,
-                               ('--interface', self._get_transport(),
-                                '--op', 'new'))
+            out_new, err_new = self._run_iscsiadm(connection_properties,
+                                                  ('--interface',
+                                                   self._get_transport(),
+                                                   '--op', 'new'),
+                                                  check_exit_code=(0, 6))
+            if err_new:
+                # retry if iscsiadm returns 6 for "database failure"
+                LOG.debug("Retrying to connect to iSCSI portal %s", portal)
+                msg = (_("Encountered database failure for %s.") % (portal))
+                raise exception.BrickException(msg=msg)
+
         # Try to set the scan mode to manual
         res = self._iscsiadm_update(connection_properties,
                                     'node.session.scan', 'manual',
