@@ -17,7 +17,6 @@ import os.path
 import re
 import time
 
-from oslo_concurrency import lockutils
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 
@@ -28,7 +27,6 @@ try:
     from os_brick.initiator.connectors import nvmeof_agent
 except ImportError:
     nvmeof_agent = None
-from os_brick.privileged import nvmeof as priv_nvme
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick import utils
 
@@ -36,14 +34,14 @@ DEV_SEARCH_PATH = '/dev/'
 
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 5
 
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
-
 LOG = logging.getLogger(__name__)
 
 
 class NVMeOFConnector(base.BaseLinuxConnector):
     """Connector class to attach/detach NVMe-oF volumes."""
 
+    native_multipath_supported = None
+
     def __init__(self, root_helper, driver=None, use_multipath=False,
                  device_scan_attempts=DEVICE_SCAN_ATTEMPTS_DEFAULT,
                  *args, **kwargs):
@@ -53,6 +51,10 @@ class NVMeOFConnector(base.BaseLinuxConnector):
             device_scan_attempts=device_scan_attempts,
             *args, **kwargs)
         self.use_multipath = use_multipath
+        self._set_native_multipath_supported()
+        if self.use_multipath and not \
+                NVMeOFConnector.native_multipath_supported:
+            LOG.warning('native multipath is not enabled')
 
     @staticmethod
     def get_search_path():
@@ -65,11 +67,12 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         volume_replicas = connection_properties.get('volume_replicas')
         if not volume_replicas:  # compatibility
             return []
+        replica_count = connection_properties.get('replica_count')
 
         try:
-            if volume_replicas and len(volume_replicas) > 1:
+            if volume_replicas and replica_count > 1:
                 return ['/dev/md/' + connection_properties.get('alias')]
-            if volume_replicas and len(volume_replicas) == 1:
+            if volume_replicas and replica_count == 1:
                 return [NVMeOFConnector.get_nvme_device_path(
                     self, volume_replicas[0]['target_nqn'],
                     volume_replicas[0]['vol_uuid'])]
@@ -99,18 +102,18 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         execute = kwargs.get('execute') or priv_rootwrap.execute
         nvmf = NVMeOFConnector(root_helper=root_helper, execute=execute)
         ret = {}
-
         nqn = None
         uuid = nvmf._get_host_uuid()
         suuid = nvmf._get_system_uuid()
         if cls.nvme_present():
-            nqn = nvmf._get_host_nqn()
+            nqn = utils.get_host_nqn()
         if uuid:
             ret['uuid'] = uuid
         if suuid:
             ret['system uuid'] = suuid  # compatibility
         if nqn:
             ret['nqn'] = nqn
+        ret['nvme_native_multipath'] = cls._set_native_multipath_supported()
         return ret
 
     def _get_host_uuid(self):
@@ -128,16 +131,6 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 "Process execution error in _get_host_uuid: %s" % str(e))
             return None
 
-    def _get_host_nqn(self):
-        try:
-            with open('/etc/nvme/hostnqn', 'r') as f:
-                host_nqn = f.read().strip()
-        except IOError:
-            host_nqn = priv_nvme.create_hostnqn()
-        except Exception:
-            host_nqn = None
-        return host_nqn
-
     def _get_system_uuid(self):
         # RSD requires system_uuid to let Cinder RSD Driver identify
         # Nova node for later RSD volume attachment.
@@ -158,6 +151,22 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 out = ""
         return out.strip()
 
+    @classmethod
+    def _set_native_multipath_supported(cls):
+        if cls.native_multipath_supported is None:
+            cls.native_multipath_supported = \
+                cls._is_native_multipath_supported()
+        return cls.native_multipath_supported
+
+    @staticmethod
+    def _is_native_multipath_supported():
+        try:
+            with open('/sys/module/nvme_core/parameters/multipath', 'rt') as f:
+                return f.read().strip() == 'Y'
+        except Exception:
+            LOG.warning("Could not find nvme_core/parameters/multipath")
+        return False
+
     def _get_nvme_devices(self):
         nvme_devices = []
         # match nvme devices like /dev/nvme10n10
@@ -317,7 +326,8 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         return self._is_nvme_available(nvme_name)
 
     @utils.trace
-    @synchronized('connect_volume', external=True)
+    @utils.connect_volume_prepare_result
+    @base.synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties):
         """Discover and attach the volume.
 
@@ -331,7 +341,7 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         :returns: dict
         """
         if connection_properties.get('vol_uuid'):  # compatibility
-            return self._connect_volume_replicated(connection_properties)
+            return self._connect_volume_by_uuid(connection_properties)
 
         current_nvme_devices = self._get_nvme_devices()
         device_info = {'type': 'block'}
@@ -368,7 +378,8 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @synchronized('connect_volume', external=True)
+    @base.synchronized('connect_volume', external=True)
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Flush the volume.
@@ -412,7 +423,8 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 raise
 
     @utils.trace
-    @synchronized('extend_volume', external=True)
+    @base.synchronized('extend_volume', external=True)
+    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
@@ -435,7 +447,7 @@ class NVMeOFConnector(base.BaseLinuxConnector):
             raise exception.VolumePathsNotFound()
 
     @utils.trace
-    def _connect_volume_replicated(self, connection_properties):
+    def _connect_volume_by_uuid(self, connection_properties):
         """connect to volume on host
 
         connection_properties for NVMe-oF must include:
@@ -443,8 +455,8 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         target_nqn - NVMe-oF Qualified Name
         vol_uuid - UUID for volume/replica
         """
-
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
         volume_alias = connection_properties.get('alias')
 
         if volume_replicas:
@@ -463,9 +475,9 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 raise exception.VolumeDeviceNotFound(
                     device=volume_replicas)
 
-            if len(volume_replicas) > 1:
+            if replica_count > 1:
                 device_path = self._handle_replicated_volume(
-                    host_device_paths, volume_alias, len(volume_replicas))
+                    host_device_paths, volume_alias, replica_count)
             else:
                 device_path = self._handle_single_replica(
                     host_device_paths, volume_alias)
@@ -485,14 +497,15 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                                       force=False, ignore_errors=False):
         device_path = None
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
         if device_info and device_info.get('path'):
             device_path = device_info['path']
         elif connection_properties.get('device_path'):
             device_path = connection_properties['device_path']
-        elif volume_replicas and len(volume_replicas) > 1:
+        elif volume_replicas and replica_count > 1:
             device_path = '/dev/md/' + connection_properties['alias']
 
-        if volume_replicas and len(volume_replicas) > 1:
+        if volume_replicas and replica_count > 1:
             NVMeOFConnector.end_raid(self, device_path)
         else:
             if self._get_fs_type(device_path) == 'linux_raid_member':
@@ -500,12 +513,15 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
     def _extend_volume_replicated(self, connection_properties):
         volume_replicas = connection_properties.get('volume_replicas')
+        replica_count = connection_properties.get('replica_count')
 
-        if volume_replicas and len(volume_replicas) > 1:
+        if volume_replicas and replica_count > 1:
             device_path = '/dev/md/' + connection_properties['alias']
             NVMeOFConnector.run_mdadm(
                 self, ['mdadm', '--grow', '--size', 'max', device_path])
         else:
+            target_nqn = None
+            vol_uuid = None
             if not volume_replicas:
                 target_nqn = connection_properties['target_nqn']
                 vol_uuid = connection_properties['vol_uuid']
@@ -518,42 +534,43 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         return self._linuxscsi.get_device_size(device_path)
 
     def _connect_target_volume(self, target_nqn, vol_uuid, portals):
-        try:
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-        except exception.VolumeDeviceNotFound:
-            host_device_path = None
-
-        if not host_device_path:
-            any_connect = NVMeOFConnector.connect_to_portals(
-                self, target_nqn, portals)
-            if not any_connect:
-                LOG.error(
-                    "No successful connections: %(host_devices)s",
-                    {'host_devices': target_nqn})
-                raise exception.VolumeDeviceNotFound(device=target_nqn)
-
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-            if not host_device_path:
-                LOG.error(
-                    "No accessible volume device: %(host_devices)s",
-                    {'host_devices': target_nqn})
-                raise exception.VolumeDeviceNotFound(device=target_nqn)
-        else:
-            NVMeOFConnector.rescan(self, target_nqn, vol_uuid)
-            host_device_path = NVMeOFConnector.get_nvme_device_path(
-                self, target_nqn, vol_uuid)
-
-        return host_device_path
+        nvme_ctrls = NVMeOFConnector.rescan(self, target_nqn)
+        any_new_connect = NVMeOFConnector.connect_to_portals(self, target_nqn,
+                                                             portals,
+                                                             nvme_ctrls)
+        if not any_new_connect and len(nvme_ctrls) == 0:
+            # no new connections and any pre-exists controllers
+            LOG.error("No successful connections to: %s", target_nqn)
+            raise exception.VolumeDeviceNotFound(device=target_nqn)
+        if any_new_connect:
+            # new connections - refresh controllers map
+            nvme_ctrls = \
+                NVMeOFConnector.get_live_nvme_controllers_map(self, target_nqn)
+        nvme_ctrls_values = list(nvme_ctrls.values())
+        dev_path = NVMeOFConnector.get_nvme_device_path(self, target_nqn,
+                                                        vol_uuid,
+                                                        nvme_ctrls_values)
+        if not dev_path:
+            LOG.error("Target %s volume %s not found", target_nqn, vol_uuid)
+            raise exception.VolumeDeviceNotFound(device=vol_uuid)
+        return dev_path
 
     @staticmethod
-    def connect_to_portals(executor, target_nqn, target_portals):
-        """connect to any of NVMe-oF target portals"""
-        any_connect = False
+    def connect_to_portals(executor, target_nqn, target_portals, nvme_ctrls):
+        # connect to any of NVMe-oF target portals -
+        # check if the controller exist before trying to connect
+        # in multipath connect all given target portals
+        any_new_connect = False
+        no_multipath = (not executor.use_multipath
+                        or not NVMeOFConnector.native_multipath_supported)
         for portal in target_portals:
             portal_address = portal[0]
             portal_port = portal[1]
+            if NVMeOFConnector.is_portal_connected(portal_address, portal_port,
+                                                   nvme_ctrls):
+                if no_multipath:
+                    break
+                continue
             if portal[2] == 'RoCEv2':
                 portal_transport = 'rdma'
             else:
@@ -563,14 +580,30 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 portal_transport, '-n', target_nqn, '-Q', '128', '-l', '-1')
             try:
                 NVMeOFConnector.run_nvme_cli(executor, nvme_command)
-                any_connect = True
-                break
+                any_new_connect = True
+                if no_multipath:
+                    break
             except Exception:
                 LOG.exception("Could not connect to portal %s", portal)
-        return any_connect
+        return any_new_connect
 
     @staticmethod
-    def _get_nvme_controller(executor, target_nqn):
+    def is_portal_connected(portal_address, portal_port, nvme_ctrls):
+        address = f"traddr={portal_address},trsvcid={portal_port}"
+        return address in nvme_ctrls
+
+    @staticmethod
+    def get_nvme_controllers(executor, target_nqn):
+        nvme_controllers = \
+            NVMeOFConnector.get_live_nvme_controllers_map(executor, target_nqn)
+        if len(nvme_controllers) > 0:
+            return nvme_controllers.values()
+        raise exception.VolumeDeviceNotFound(device=target_nqn)
+
+    @staticmethod
+    def get_live_nvme_controllers_map(executor, target_nqn):
+        """returns map of all live controllers and their addresses """
+        nvme_controllers = dict()
         ctrls = glob.glob('/sys/class/nvme-fabrics/ctl/nvme*')
         for ctrl in ctrls:
             try:
@@ -582,32 +615,45 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                         state, _err = executor._execute(
                             'cat', ctrl + '/state', run_as_root=True,
                             root_helper=executor._root_helper)
-                        if 'live' not in state:
+                        if 'live' in state:
+                            address_file = ctrl + '/address'
+                            try:
+                                with open(address_file, 'rt') as f:
+                                    address = f.read().strip()
+                            except Exception:
+                                LOG.warning("Failed to read file %s",
+                                            address_file)
+                                continue
+                            ctrl_name = os.path.basename(ctrl)
+                            LOG.debug("[!] address: %s|%s", address, ctrl_name)
+                            nvme_controllers[address] = ctrl_name
+                        else:
                             LOG.debug("nvmeof ctrl device not live: %s", ctrl)
-                            raise exception.VolumeDeviceNotFound(device=ctrl)
-                        return ctrl[ctrl.rfind('/') + 1:]
             except putils.ProcessExecutionError as e:
                 LOG.exception(e)
-
-        raise exception.VolumeDeviceNotFound(device=target_nqn)
+        return nvme_controllers
 
     @staticmethod
-    @utils.retry(exception.VolumeDeviceNotFound)
-    def get_nvme_device_path(executor, target_nqn, vol_uuid):
-        nvme_ctrl = NVMeOFConnector._get_nvme_controller(executor, target_nqn)
-        try:
-            blocks = glob.glob(
-                '/sys/class/nvme-fabrics/ctl/' + nvme_ctrl +
-                '/' + nvme_ctrl + 'n*')
-            for block in blocks:
-                uuid_lines, _err = executor._execute(
-                    'cat', block + '/uuid', run_as_root=True,
-                    root_helper=executor._root_helper)
-                if uuid_lines.split('\n')[0] == vol_uuid:
-                    return '/dev/' + block[block.rfind('/') + 1:]
-        except putils.ProcessExecutionError as e:
-            LOG.exception(e)
-
+    @utils.retry(exception.VolumeDeviceNotFound, retries=5)
+    def get_nvme_device_path(executor, target_nqn, vol_uuid, nvme_ctrls=None):
+        if not nvme_ctrls:
+            nvme_ctrls = NVMeOFConnector.get_nvme_controllers(executor,
+                                                              target_nqn)
+        LOG.debug("[!] nvme_ctrls: %s", nvme_ctrls)
+        for nvme_ctrl in nvme_ctrls:
+            uuid_paths = glob.glob('/sys/class/block/' + nvme_ctrl + 'n*/uuid')
+            for uuid_path in uuid_paths:
+                try:
+                    uuid_lines, _err = executor._execute(
+                        'cat', uuid_path, run_as_root=True,
+                        root_helper=executor._root_helper)
+                    if uuid_lines.split('\n')[0] == vol_uuid:
+                        ignore = len('/uuid')
+                        ns_ind = uuid_path.rfind('/', 0, -ignore)
+                        nvme_device = uuid_path[ns_ind + 1: -ignore]
+                        return '/dev/' + nvme_device
+                except putils.ProcessExecutionError as e:
+                    LOG.exception(e)
         raise exception.VolumeDeviceNotFound(device=vol_uuid)
 
     def _handle_replicated_volume(self, host_device_paths,
@@ -780,6 +826,19 @@ class NVMeOFConnector(base.BaseLinuxConnector):
 
         LOG.debug('[!] cmd = ' + str(cmd))
         NVMeOFConnector.run_mdadm(executor, cmd)
+        # sometimes under load, md is not created right away, so we wait
+        for i in range(60):
+            try:
+                is_exist = os.path.exists("/dev/md/" + name)
+                LOG.debug("[!] md is_exist = %s", is_exist)
+                if is_exist:
+                    return
+                time.sleep(1)
+            except Exception:
+                LOG.debug('[!] Exception_wait_raid!')
+        msg = _("md: /dev/md/%s not found.") % name
+        LOG.error(msg)
+        raise exception.NotFound(message=msg)
 
     @staticmethod
     def end_raid(executor, device_path):
@@ -788,12 +847,11 @@ class NVMeOFConnector(base.BaseLinuxConnector):
             for i in range(10):
                 try:
                     cmd_out = NVMeOFConnector.stop_raid(
-                        executor, device_path)
+                        executor, device_path, True)
                     if not cmd_out:
                         break
                 except Exception:
-                    break
-                time.sleep(1)
+                    time.sleep(1)
             try:
                 is_exist = os.path.exists(device_path)
                 LOG.debug("[!] is_exist = %s", is_exist)
@@ -804,10 +862,10 @@ class NVMeOFConnector(base.BaseLinuxConnector):
                 LOG.debug('[!] Exception_stop_raid!')
 
     @staticmethod
-    def stop_raid(executor, md_path):
+    def stop_raid(executor, md_path, raise_exception=False):
         cmd = ['mdadm', '--stop', md_path]
         LOG.debug("[!] cmd = " + str(cmd))
-        cmd_out = NVMeOFConnector.run_mdadm(executor, cmd)
+        cmd_out = NVMeOFConnector.run_mdadm(executor, cmd, raise_exception)
         return cmd_out
 
     @staticmethod
@@ -846,27 +904,26 @@ class NVMeOFConnector(base.BaseLinuxConnector):
         return out, err
 
     @staticmethod
-    def rescan(executor, target_nqn, vol_uuid):
-        ctr_device = (
-            NVMeOFConnector.get_search_path() +
-            NVMeOFConnector._get_nvme_controller(executor, target_nqn))
-        nvme_command = ('ns-rescan', ctr_device)
-        try:
-            NVMeOFConnector.run_nvme_cli(executor, nvme_command)
-        except Exception as e:
-            raise exception.CommandExecutionFailed(e, cmd=nvme_command)
+    def rescan(executor, target_nqn):
+        nvme_ctrls = NVMeOFConnector.get_live_nvme_controllers_map(executor,
+                                                                   target_nqn)
+        for nvme_ctrl in nvme_ctrls.values():
+            ctr_device = (NVMeOFConnector.get_search_path() + nvme_ctrl)
+            nvme_command = ('ns-rescan', ctr_device)
+            try:
+                NVMeOFConnector.run_nvme_cli(executor, nvme_command)
+            except Exception as e:
+                LOG.exception(e)
+        return nvme_ctrls
 
     def _get_fs_type(self, device_path):
-        cmd = ['blkid', device_path, '-s', 'TYPE', '-o', 'value']
+        cmd = ('blkid', device_path, '-s', 'TYPE', '-o', 'value')
         LOG.debug("[!] cmd = " + str(cmd))
         fs_type = None
 
-        try:
-            lines, err = self._execute(
-                *cmd, run_as_root=True, root_helper=self._root_helper)
-
-            fs_type = lines.split('\n')[0]
-        except putils.ProcessExecutionError:
-            return None
-
-        return fs_type
+        # We don't care about errors, on error lines will be '' so it's ok
+        lines, err = self._execute(
+            *cmd, run_as_root=True, root_helper=self._root_helper,
+            check_exit_code=False)
+        fs_type = lines.split('\n')[0]
+        return fs_type or None
