@@ -16,7 +16,6 @@ import json
 import os
 import urllib
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
 import requests
 
@@ -30,7 +29,6 @@ from os_brick import utils
 LOG = logging.getLogger(__name__)
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 3
 CONNECTOR_CONF_PATH = '/opt/emc/scaleio/openstack/connector.conf'
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
 
 
 def io(_type, nr):
@@ -75,6 +73,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.volume_path = None
         self.iops_limit = None
         self.bandwidth_limit = None
+        self.verify_certificate = None
+        self.certificate_path = None
 
     def _get_guid(self):
         try:
@@ -204,7 +204,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         r = requests.get(
             request,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request)
@@ -250,7 +250,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         r = requests.get(request,
                          auth=(self.server_username, self.server_token),
-                         verify=False)
+                         verify=self._verify_cert())
 
         r = self._check_response(r, request)
 
@@ -290,7 +290,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             r = requests.get(
                 login_request,
                 auth=(self.server_username, self.server_password),
-                verify=False
+                verify=self._verify_cert()
             )
 
             token = r.json()
@@ -301,7 +301,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             if is_get_request:
                 res = requests.get(request,
                                    auth=(self.server_username, token),
-                                   verify=False)
+                                   verify=self._verify_cert())
             else:
                 headers = {'content-type': 'application/json'}
                 res = requests.post(
@@ -309,7 +309,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                     data=json.dumps(params),
                     headers=headers,
                     auth=(self.server_username, token),
-                    verify=False
+                    verify=self._verify_cert()
                 )
 
             self.server_token = token
@@ -317,6 +317,12 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
         return response
 
+    def _verify_cert(self):
+        verify_cert = self.verify_certificate
+        if self.verify_certificate and self.certificate_path:
+            verify_cert = self.certificate_path
+        return verify_cert
+
     def get_config(self, connection_properties):
         self.local_sdc_ip = connection_properties['hostIP']
         self.volume_name = connection_properties['scaleIO_volname']
@@ -331,12 +337,17 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             connection_properties)
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
+        self.verify_certificate = (
+            connection_properties.get('verify_certificate')
+        )
+        self.certificate_path = connection_properties.get('certificate_path')
         device_info = {'type': 'block',
                        'path': self.volume_path}
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
+    @utils.connect_volume_prepare_result
+    @base.synchronized('scaleio', 'scaleio-', external=True)
     def connect_volume(self, connection_properties):
         """Connect the volume.
 
@@ -382,7 +393,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -431,7 +442,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 data=json.dumps(params),
                 headers=headers,
                 auth=(self.server_username, self.server_token),
-                verify=False
+                verify=self._verify_cert()
             )
             r = self._check_response(r, request, False, params)
             if r.status_code != self.OK_STATUS_CODE:
@@ -450,7 +461,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
+    @base.synchronized('scaleio', 'scaleio-', external=True)
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Disconnect the ScaleIO volume.
@@ -495,7 +507,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             data=json.dumps(params),
             headers=headers,
             auth=(self.server_username, self.server_token),
-            verify=False
+            verify=self._verify_cert()
         )
 
         r = self._check_response(r, request, False, params)
@@ -514,6 +526,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
                 LOG.error(msg)
                 raise exception.BrickException(message=msg)
 
+    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
