@@ -28,6 +28,7 @@ from os_brick.initiator.connectors import nvmeof
 from os_brick.initiator import linuxfc
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests import base as test_base
+from os_brick import utils
 
 MY_IP = '10.0.0.1'
 FAKE_SCSI_WWN = '1234567890'
@@ -41,11 +42,14 @@ class ZeroIntervalLoopingCall(loopingcall.FixedIntervalLoopingCall):
 
 class ConnectorUtilsTestCase(test_base.TestCase):
 
+    @mock.patch.object(nvmeof.NVMeOFConnector,
+                       '_is_native_multipath_supported',
+                       return_value=False)
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_system_uuid',
                        return_value=None)
     @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_uuid',
                        return_value=None)
-    @mock.patch.object(nvmeof.NVMeOFConnector, '_get_host_nqn',
+    @mock.patch.object(utils, 'get_host_nqn',
                        return_value=None)
     @mock.patch.object(iscsi.ISCSIConnector, 'get_initiator',
                        return_value='fakeinitiator')
@@ -63,6 +67,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                                              mock_nqn,
                                              mock_hostuuid,
                                              mock_sysuuid,
+                                             mock_native_multipath_supported,
                                              host='fakehost'):
         props_actual = connector.get_connector_properties('sudo',
                                                           MY_IP,
@@ -75,6 +80,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                  'host': host,
                  'ip': MY_IP,
                  'multipath': multipath_result,
+                 'nvme_native_multipath': False,
                  'os_type': os_type,
                  'platform': platform,
                  'do_local_attach': False}
